import requests

from requests.exceptions import RequestException

from .authentication import Authentication
from .exceptions import (StampsError, InvalidRequest, ServerError,
                         AuthenticationError, ConnectionError)
from .memberships import Memberships
from .redemptions import Redemptions
from .rewards import Rewards
from .transactions import Transactions


class Client(object):

    def __init__(self, token=None, merchant_id=None,
                 store_id=None, host="https://stamps.co.id"):
        # Basic settings
        self.token = token
        self.merchant_id = merchant_id
        self.store_id = store_id
        self.base_url = host + "/api"

        # Keep track of the response object for debugging purpose
        self._response = None

        # Attach innerclass
        self.authentication = Authentication(self)
        self.redemptions = Redemptions(self)
        self.transactions = Transactions(self)
        self.memberships = Memberships(self)
        self.rewards = Rewards(self)

    def _call(self, request_type, url, payload):
        try:
            if request_type == 'GET':
                self._response = requests.get(url, params=payload)
            if request_type == 'POST':
                self._response = requests.post(url, json=payload)

        except RequestException as Error:
            raise ConnectionError(None, Error)

        self._handle_api_response(self._response)

        # Return the response as dict for convenience
        return self._response.json()

    def _handle_api_response(self, response):
        # Only OK 200 should return result safely
        if response.status_code == 200:
            return

        exception_to_raise = None
        if response.status_code == 400:
            exception_to_raise = InvalidRequest
        elif response.status_code == 403:
            exception_to_raise = AuthenticationError
        else:
            exception_to_raise = ServerError

        raise exception_to_raise(response.status_code, response.json(),
                                 response=response)
